@@ -41,14 +41,7 @@ sub browse_the_table {
     $ax->print_sql_info( $ax->get_sql_info( $sql ) );
 
     PRINT_TABLE: while ( 1 ) {
-        my $all_arrayref;
-        if ( ! eval {
-            ( $all_arrayref, $sql ) = $sf->__on_table( $sql, $changed );
-            1 }
-        ) {
-            $ax->print_error_message( $@ );
-            last PRINT_TABLE;
-        }
+        ( my $all_arrayref, $sql ) = $sf->__on_table( $sql, $changed );
         if ( ! defined $all_arrayref ) {
             last PRINT_TABLE;
         }
@@ -114,86 +107,52 @@ sub __on_table {
             }
             $old_idx = $idx;
         }
-        my $backup_sql = $ax->backup_href( $sql );
         if ( $sub_stmt eq $select ) {
             my $ret = $sb->select( $sql );
-            if ( ! defined $ret ) {
-                $sql = $backup_sql;
-            }
-            else {
-                $changed->{$sub_stmt} = $ret;
-            }
+            $changed->{$sub_stmt} = $ret;
         }
         elsif ( $sub_stmt eq $distinct ) {
             my $ret = $sb->distinct( $sql );
-            if ( ! defined $ret ) {
-                $sql = $backup_sql;
-            }
-            else {
-                $changed->{$sub_stmt} = $ret;
-            }
+            $changed->{$sub_stmt} = $ret;
         }
         elsif ( $sub_stmt eq $aggregate ) {
             my $ret = $sb->aggregate( $sql );
-            if ( ! defined $ret ) {
-                $sql = $backup_sql;
-            }
-            else {
-                $changed->{$sub_stmt} = $ret;
-            }
+            $changed->{$sub_stmt} = $ret;
         }
         elsif ( $sub_stmt eq $where ) {
             my $ret = $sb->where( $sql );
-            if ( ! defined $ret ) {
-                $sql = $backup_sql;
-            }
-            else {
-                $changed->{$sub_stmt} = $ret;
-            }
+            $changed->{$sub_stmt} = $ret;
         }
         elsif ( $sub_stmt eq $group_by ) {
             my $ret = $sb->group_by( $sql );
-            if ( ! defined $ret ) {
-                $sql = $backup_sql;
-            }
-            else {
-                $changed->{$sub_stmt} = $ret;
-            }
+            $changed->{$sub_stmt} = $ret;
         }
         elsif ( $sub_stmt eq $having ) {
             my $ret = $sb->having( $sql );
-            if ( ! defined $ret ) {
-                $sql = $backup_sql;
-            }
-            else {
-                $changed->{$sub_stmt} = $ret;
-            }
+            $changed->{$sub_stmt} = $ret;
         }
         elsif ( $sub_stmt eq $order_by ) {
             my $ret = $sb->order_by( $sql );
-            if ( ! defined $ret ) {
-                $sql = $backup_sql;
-            }
-            else {
-                $changed->{$sub_stmt} = $ret;
-            }
+            $changed->{$sub_stmt} = $ret;
         }
         elsif ( $sub_stmt eq $limit ) {
             my $ret = $sb->limit_offset( $sql );
-            if ( ! defined $ret ) {
-                $sql = $backup_sql;
-            }
-            else {
-                $changed->{$sub_stmt} = $ret;
-            }
+            $changed->{$sub_stmt} = $ret;
         }
         elsif ( $sub_stmt eq $hidden ) {
-            require App::DBBrowser::Table::InsertUpdateDelete;
-            my $write = App::DBBrowser::Table::InsertUpdateDelete->new( $sf->{i}, $sf->{o}, $sf->{d} );
-            $write->table_write_access( $sql );
+            if ( ! eval {
+                require App::DBBrowser::Table::InsertUpdateDelete;
+                my $write = App::DBBrowser::Table::InsertUpdateDelete->new( $sf->{i}, $sf->{o}, $sf->{d} );
+                require Clone;
+                my $backup_sql = Clone::clone( $sql );
+                $write->table_write_access( $sql );
+                $sql = $backup_sql;
+                1 }
+            ) {
+                $ax->print_error_message( $@ );
+            }
             $sf->{d}{stmt_types} = [ 'Select' ];
             $old_idx = 1;
-            $sql = $backup_sql; # so no need for table_write_access to return $sql
         }
         elsif ( $sub_stmt eq $export ) {
             my $file_fs = $sf->__get_filename_fs( $sql );
@@ -234,7 +193,14 @@ sub __on_table {
             print hide_cursor(); # safety
             print clear_screen();
             print 'Computing:' . "\r" if $sf->{o}{table}{progress_bar};
-            my $all_arrayref = $sf->__selected_statement_result( $sql );
+            my $all_arrayref;
+            if ( ! eval {
+                $all_arrayref = $sf->__selected_statement_result( $sql );
+                1 }
+            ) {
+                $ax->print_error_message( $@ );
+                next CUSTOMIZE;
+            }
             # return $sql explicitly since after a restore-backup $sql refers to a different hash.
             return $all_arrayref, $sql;
         }
